﻿/* *********************************************************************
* Copyright (c) 1986-2018 ICONICS, Inc.
* Contains Confidential information
* THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
* **********************************************************************/

/* *********************************************************************
* This file is subject to the terms and conditions defined in 
* 'ICONICS Extensibility Toolkit LICENSE.pdf'
* which is part of this source code package.
* **********************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

using Ico.Fwx.Communication.Client;

using Ico.Workflow;
using Ico.Workflow.ComponentModel;
using Ico.Workflow.Runtime.Diagrams;
using Ico.Workflow.Activities.Custom;

using Ico.Workflow.Misc;
using Ico.DataFlow.Schemas;

namespace MyCustomActivity1
{
    /// <summary>
    /// This class represents special custom BWX64 activity type.
    /// As an example, it will truncate a specified file (i.e. it would delete its content, leaving the file of size 0)
    /// </summary>

    [CustomActivityEditor("MyCustomActivity1.Design.MyActivity1Editor, MyCustomActivity1Design")]  //assign editor for this activity
    [CustomActivityIcon(ActivityIcon.Cube)]     //somtething to display in the designer
    public class MyActivity1 : CustomActivityBase //make sure to use this as a base class
    {
        /// <summary>
        /// Override this Property to return the name of your company
        /// </summary>
        public override string CompanyName
        {
            get
            {
                return "The Charging Mirash";
            }
        }

        /// <summary>
        ///   Override this Property to return your Product Name
        /// </summary>
        public override string ProductName
        {
            get
            {
                return "Elbonai Ultra Lux";
            }
        }

        /// <summary>
        ///  Override this Property to return your Activity Version
        /// </summary>
        public override string ActivityVersion
        {
            get
            {
                return "1.0.0.1";
            }
        }

        /// <summary>
        /// Override this Property to return your Activity's Category
        /// </summary>
        public override ActivityCategory Category
        {
            get
            {
                return ActivityCategory.UserDefined;
            }
        }

        /// <summary>
        ///  Override this Property to return your Activity's type name. It will be displayed as Activity Type.
        /// </summary>
        public override string TypeName
        {
            get
            {
                return "My Cool Custom Activity #1";
            }
        }

        /// <summary>
        /// Override this Property to return your Activity's type description.
        /// </summary>
        public override string TypeDescription
        {
            get
            {
                return "This activity was designed for educational purpose. At runtime it will truncate a given file.";
            }
        }


        //*******************************************************************************************************************************
        // ADD ANY PROPERTIES YOU WANT. They must be serializable or be an arrays of serializable types. Please provide both set and get.
        //*******************************************************************************************************************************

        /// <summary>
        /// Gets, Sets FilePath that this activity will truncate.
        /// The value will be set by our custom configurator and serialized by Workbench Diagram Editor
        /// </summary>
        public string FilePath
        {
            get
            { return _filePath; }

            set
            {
                if (_filePath != value)
                {
                    _filePath = value;
                    OnPropertyChanged("FilePath");
                }
            }
        }

        private string _filePath = "";


        /// <summary>
        /// Here is the code that executes our Activity's work. It should return one of ActivityResult values.
        /// In this example we truncate a file.
        /// </summary>
        /// <param name="cancToken"></param>
        /// <returns></returns>
        protected override async Task<ActivityResult> DoWork(CancellationToken cancToken)
        {
            //do not do anything if current transaction was cancelled for any reasons
            if (cancToken.IsCancellationRequested)
                return ActivityResult.Cancelled;

            //does configuration data make sense?
            if (string.IsNullOrEmpty(FilePath))
            {
                LastError = "FilePath not set!";
                return ActivityResult.Failed;
            }

            //this would resolve global aliases, language aliases or parameters in the FilePath, if any existed.
            string resolvedFilePath = await ResolveParameter(FilePath);

            if(!System.IO.File.Exists(resolvedFilePath))
            {
                LastError = "FilePath " + resolvedFilePath + " does not exist!";
                return ActivityResult.Failed;
            }

            //execute required work
            try
            {
                //open file in Truncate mode and do nothing more with it
                using (System.IO.FileStream f = new System.IO.FileStream(resolvedFilePath, System.IO.FileMode.Truncate))
                {
                    return ActivityResult.Succeeded;
                }
            }
            catch (Exception ex)
            {
                //oops, failed miserably
                LastError = string.Format("Exception at truncating file {0}: {1}", resolvedFilePath, ex.Message);
                return ActivityResult.Failed;
            }
        }
    }
}
